@@ -114,8 +114,8 @@ use CaseConvention::{Camel, Pascal};
 // This enum is used to distinguish between the ingress and egress versions of a type.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 enum Direction {
-    Out,
     In,
+    Out,
 }
 
 use Direction::{In, Out};
@@ -281,12 +281,14 @@ function deserializeVarint(
 ): [number, bigint] {{
   const firstByte = dataView.getUint8(offset);
   const sizeMinusOne = varintSizeFromFirstByte(firstByte) - 1;
+
   const offsetPlusOne = offset + 1;
   dataView64.setBigUint64(0, 0n, true);
   for (let i = 0; i < sizeMinusOne; ++i) {{
     dataView64.setUint8(i, dataView.getUint8(offsetPlusOne + i));
   }}
   const remainingBytesValue = dataView64.getBigUint64(0, true);
+
   switch (sizeMinusOne) {{
     case 0:
       return [offset + 1, BigInt(firstByte >> 1)];
@@ -376,6 +378,7 @@ function serializeFieldHeader(
           offset,
           (index << 2n) | BigInt(0b11),
         );
+
         return serializeVarint(dataView, offset, BigInt(payloadSize));
       }}
   }}
@@ -759,8 +762,8 @@ fn write_schema<T: Write>(
                         } else {
                             write!(buffer, ", ")?;
                         }
+                        write!(buffer, "$")?;
                         write_identifier(buffer, &field.name, Camel, None)?;
-                        write!(buffer, "Field")?;
                     }
                     writeln!(buffer, ";")?;
                     writeln!(buffer)?;
@@ -820,8 +823,9 @@ fn write_schema<T: Write>(
                         write_indentation(buffer, indentation + 5)?;
                         writeln!(buffer, "offset += oldOffset;")?;
                         write_indentation(buffer, indentation + 5)?;
+                        write!(buffer, "$")?;
                         write_identifier(buffer, &field.name, Camel, None)?;
-                        writeln!(buffer, "Field = payload;")?;
+                        writeln!(buffer, " = payload;")?;
                         write_indentation(buffer, indentation + 5)?;
                         writeln!(buffer, "break;")?;
                         write_indentation(buffer, indentation + 4)?;
@@ -854,8 +858,9 @@ fn write_schema<T: Write>(
                                     } else {
                                         write!(buffer, " || ")?;
                                     }
+                                    write!(buffer, "$")?;
                                     write_identifier(buffer, &field.name, Camel, None)?;
-                                    write!(buffer, "Field === undefined")?;
+                                    write!(buffer, " === undefined")?;
                                 }
                             }
                         }
@@ -879,9 +884,9 @@ fn write_schema<T: Write>(
                 for field in &declaration.fields {
                     write_indentation(buffer, indentation + 4)?;
                     write_identifier(buffer, &field.name, Camel, None)?;
-                    write!(buffer, ": ")?;
+                    write!(buffer, ": $")?;
                     write_identifier(buffer, &field.name, Camel, None)?;
-                    writeln!(buffer, "Field,")?;
+                    writeln!(buffer, ",")?;
                 }
                 write_indentation(buffer, indentation + 3)?;
                 writeln!(buffer, "}}")?;
@@ -947,7 +952,7 @@ fn write_schema<T: Write>(
                     writeln!(buffer, "let payloadSize = 0;")?;
                     writeln!(buffer)?;
                     write_indentation(buffer, indentation + 2)?;
-                    writeln!(buffer, "switch (value.field) {{")?;
+                    writeln!(buffer, "switch (value.$field) {{")?;
                     for field in &declaration.fields {
                         write_indentation(buffer, indentation + 3)?;
                         write!(buffer, "case '")?;
@@ -957,7 +962,9 @@ fn write_schema<T: Write>(
                         if matches!(field.r#type.variant, schema::TypeVariant::Unit) {
                             writeln!(buffer, "const payload = null;")?;
                         } else {
-                            writeln!(buffer, "const payload = value.value;")?;
+                            write!(buffer, "const payload = value.")?;
+                            write_identifier(buffer, &field.name, Camel, None)?;
+                            writeln!(buffer, ";")?;
                         }
                         write_size_calculation(
                             buffer,
@@ -976,7 +983,7 @@ fn write_schema<T: Write>(
                         )?;
                         match field.rule {
                             schema::Rule::Asymmetric | schema::Rule::Optional => {
-                                writeln!(buffer, " + size(value.fallback);")?;
+                                writeln!(buffer, " + size(value.$fallback);")?;
                             }
                             schema::Rule::Required => {
                                 writeln!(buffer, ";")?;
@@ -1016,7 +1023,7 @@ fn write_schema<T: Write>(
                     writeln!(buffer, "let payloadSize = 0;")?;
                     writeln!(buffer)?;
                     write_indentation(buffer, indentation + 2)?;
-                    writeln!(buffer, "switch (value.field) {{")?;
+                    writeln!(buffer, "switch (value.$field) {{")?;
                     for field in &declaration.fields {
                         write_indentation(buffer, indentation + 3)?;
                         write!(buffer, "case '")?;
@@ -1026,7 +1033,9 @@ fn write_schema<T: Write>(
                         if matches!(field.r#type.variant, schema::TypeVariant::Unit) {
                             writeln!(buffer, "const payload = null;")?;
                         } else {
-                            writeln!(buffer, "const payload = value.value;")?;
+                            write!(buffer, "const payload = value.")?;
+                            write_identifier(buffer, &field.name, Camel, None)?;
+                            writeln!(buffer, ";")?;
                         }
                         write_size_calculation(
                             buffer,
@@ -1057,7 +1066,7 @@ fn write_schema<T: Write>(
                                 write_indentation(buffer, indentation + 4)?;
                                 writeln!(
                                     buffer,
-                                    "offset = serialize(dataView, offset, value.fallback);",
+                                    "offset = serialize(dataView, offset, value.$fallback);",
                                 )?;
                             }
                             schema::Rule::Required => {}
@@ -1137,7 +1146,7 @@ fn write_schema<T: Write>(
                             write_indentation(buffer, indentation + 5)?;
                             writeln!(
                                 buffer,
-                                "const [newNewOffset, fallback] = \
+                                "const [newNewOffset, $fallback] = \
                                     deserialize(dataViewAlias, offset);",
                             )?;
                             write_indentation(buffer, indentation + 5)?;
@@ -1151,18 +1160,19 @@ fn write_schema<T: Write>(
                     write_indentation(buffer, indentation + 6)?;
                     writeln!(buffer, "{{")?;
                     write_indentation(buffer, indentation + 7)?;
-                    write!(buffer, "field: '")?;
+                    write!(buffer, "$field: '")?;
                     write_identifier(buffer, &field.name, Camel, None)?;
                     writeln!(buffer, "',")?;
                     if !matches!(field.r#type.variant, schema::TypeVariant::Unit) {
                         write_indentation(buffer, indentation + 7)?;
-                        writeln!(buffer, "value: payload,")?;
+                        write_identifier(buffer, &field.name, Camel, None)?;
+                        writeln!(buffer, ": payload,")?;
                     }
                     match field.rule {
                         schema::Rule::Asymmetric | schema::Rule::Required => {}
                         schema::Rule::Optional => {
                             write_indentation(buffer, indentation + 7)?;
-                            writeln!(buffer, "fallback,")?;
+                            writeln!(buffer, "$fallback,")?;
                         }
                     }
                     write_indentation(buffer, indentation + 6)?;
@@ -1230,10 +1240,10 @@ fn write_struct<T: Write>(
         write_identifier(buffer, &field.name, Camel, None)?;
         match field.rule {
             schema::Rule::Asymmetric => match direction {
-                Direction::Out => {}
                 Direction::In => {
                     write!(buffer, "?")?;
                 }
+                Direction::Out => {}
             },
             schema::Rule::Optional => {
                 write!(buffer, "?")?;
@@ -1269,30 +1279,29 @@ fn write_choice<T: Write>(
     for field in fields {
         writeln!(buffer)?;
         write_indentation(buffer, indentation + 1)?;
-        write!(buffer, "| {{ field: '")?;
+        write!(buffer, "| {{ $field: '")?;
         write_identifier(buffer, &field.name, Camel, None)?;
         write!(buffer, "'")?;
-        let fallback = match field.rule {
+
+        if !matches!(field.r#type.variant, schema::TypeVariant::Unit) {
+            write!(buffer, "; ")?;
+            write_identifier(buffer, &field.name, Camel, None)?;
+            write!(buffer, ": ")?;
+            write_type(buffer, imports, namespace, &field.r#type.variant, direction)?;
+        }
+
+        if match field.rule {
             schema::Rule::Asymmetric => match direction {
-                Direction::Out => true,
                 Direction::In => false,
+                Direction::Out => true,
             },
             schema::Rule::Optional => true,
             schema::Rule::Required => false,
-        };
-        if fallback {
-            if matches!(field.r#type.variant, schema::TypeVariant::Unit) {
-            } else {
-                write!(buffer, "; value: ")?;
-                write_type(buffer, imports, namespace, &field.r#type.variant, direction)?;
-            }
-            write!(buffer, "; fallback: ")?;
+        } {
+            write!(buffer, "; $fallback: ")?;
             write_identifier(buffer, name, Pascal, Some(direction))?;
-        } else if matches!(field.r#type.variant, schema::TypeVariant::Unit) {
-        } else {
-            write!(buffer, "; value: ")?;
-            write_type(buffer, imports, namespace, &field.r#type.variant, direction)?;
         }
+
         write!(buffer, " }}")?;
     }
 
@@ -1301,7 +1310,7 @@ fn write_choice<T: Write>(
     if fields.len() == 1 {
         writeln!(buffer)?;
         write_indentation(buffer, indentation + 1)?;
-        write!(buffer, "| {{ field: never }}")?;
+        write!(buffer, "| {{ $field: never }}")?;
     }
 
     if fields.is_empty() {
